@@ -115,18 +115,21 @@ class priColor:
     FLASH     = '\033[05m'    # 点滅
     RED_FLASH = '\033[05;41m' # 赤背景+点滅
     END       = '\033[0m'
-    
+
     @staticmethod
-    def color(value, color="RED"):
-        color = color.upper()
-        handleKeyError(priColor.__dict__.keys(), color=color)
-        return f"{priColor.__dict__[color.upper()]}{value}{priColor.END}"
+    def color(value, color=None):
+        if color is None:
+            return str(value)
+        else:
+            color = color.upper()
+            handleKeyError(priColor.__dict__.keys(), color=color)
+            return f"{priColor.__dict__[color.upper()]}{value}{priColor.END}"
 
 def handleKeyError(lst, msg_="", **kwargs):
     k,v = kwargs.popitem()
     if v not in lst:
         lst = ', '.join([f"'{e}'" for e in lst])
-        raise KeyError(f"Please chose the argment `{k}` from {lst}.\n{msg_}")
+        raise KeyError(f"Please chose the argment `{k}` from {lst}.\n\033[32m{msg_}\033[0m")
 
 def handleTypeError(types, msg_="", **kwargs):
     type2str = lambda t: re.sub(r"<class '(.*?)'>", r"\033[34m\1\033[0m", str(t))
@@ -138,7 +141,7 @@ def handleTypeError(types, msg_="", **kwargs):
             err_msg = f"must be {str_true_types}"
         else:
             err_msg = f"must be one of {str_true_types}"
-        raise TypeError(f"`{k}` {err_msg}, not {srt_false_type}.\n{msg_}")
+        raise TypeError(f"`{k}` {err_msg}, not {srt_false_type}.\n\033[32m{msg_}\033[0m")
 
 def urlDecorate(url, addDate=True):
     """ Decorate URL like Wget. (Add datetime information and coloring url to blue.) """
@@ -191,7 +194,7 @@ f_signs            = ["+", "-", " ", ""]
 f_grouping_options = ["_", ",", ""]
 f_types            = ["b", "c", "d", "e", "E", "f", "F", "g", "G", "n", "o", "s", "x", "X", "%"]
 
-def format_spec_create(width, align=">", sign="", zero_padding=False,
+def format_spec_create(width=0, align=">", sign="", zero_padding=False,
                        grouping_option="", fmt=""):
     """
     Create a function which returns a formatted text.
@@ -218,7 +221,7 @@ def format_spec_create(width, align=">", sign="", zero_padding=False,
     handleTypeError(types=[int], width=width)
     return lambda fill : f"{fill:{align}{sign}{zero}{width}{grouping_option}{fmt}}"
 
-def print_func_create(width, align=">", sign="", zero_padding=False,
+def print_func_create(width=0, align=">", sign="", zero_padding=False,
                       grouping_option="", fmt="", color="black",
                       left_side_bar="", right_side_bar="",
                       left_margin=0, right_margin=0, end="\n"):
